import javax.naming.*;
import javax.naming.directory.*;
import java.util.Hashtable;

/**
  * Demonstrates how to add a new attribute type to the schema.
  *
  * usage: java AddAttributeDefn
  */
class AddAttributeDefn {
    public static void main(String[] args) {

	// Set up environment for creating initial context
	Hashtable env = new Hashtable(11);
	env.put(Context.INITIAL_CONTEXT_FACTORY, 
	    "com.sun.jndi.ldap.LdapCtxFactory");
	env.put(Context.PROVIDER_URL, "ldap://localhost:389/o=JNDITutorial");

	// Must authenticate as directory administrator in order to update schema
	env.put(Context.SECURITY_PRINCIPAL, "cn=Directory Manager");
	env.put(Context.SECURITY_CREDENTIALS, "secret99");
	env.put(Context.SECURITY_AUTHENTICATION, "simple");

	// These are required for using with Netscape Directory Server 3.x
	env.put(Context.SECURITY_AUTHENTICATION, "simple");
	env.put("com.sun.naming.netscape.schemaBugs", "true");

	try {
	    // Specify attributes for schema object
	    Attributes attrs = new BasicAttributes(true); // ignore case
	    attrs.put("NUMERICOID", "1.3.6.1.4.1.42.2.27.4.2.3.1.1.2");
	    attrs.put("NAME", "fooAttr");
	    attrs.put("DESC", "for JNDITutorial example only");
	    attrs.put("SYNTAX", "1.3.6.1.4.1.1466.115.121.1.15");

	    // Create the initial context
	    DirContext ctx = new InitialDirContext(env);

	    // Get the schema tree root
	    DirContext schema = ctx.getSchema("");

	    // Add new schema object for "fooAttr"
    	    DirContext newAttr = schema.createSubcontext(
		"AttributeDefinition/fooAttr", attrs);

	    // See what was added
	    System.out.println(
		schema.getAttributes("AttributeDefinition/fooAttr"));
	} catch (NamingException e) {
	    e.printStackTrace();
	}
    }
}
