import javax.naming.*;
import javax.naming.directory.*;

import java.util.Hashtable;

/**
 * Demonstrates how to search using a filter expression with filter arguments.
 *
 * usage: java FilterArgs
 */
class FilterArgs {
    public static void main(String[] args) {


	// Set up environment for creating initial context
	Hashtable env = new Hashtable(11);
	env.put(Context.INITIAL_CONTEXT_FACTORY, 
	    "com.sun.jndi.ldap.LdapCtxFactory");
	env.put(Context.PROVIDER_URL, "ldap://localhost:389/o=JNDITutorial");

	try {
	    // Create initial context
	    DirContext ctx = new InitialDirContext(env);

	    // Specify filter arguments
	    byte[] key = {(byte)0x61, (byte)0x62, (byte)0x63, (byte)0x64, 
			  (byte)0x65, (byte)0x66, (byte)0x67};
	    String name = "User";

	    // Perform search
	    NamingEnumeration answer = ctx.search("ou=NewHires", 
		"(&(mySpecialKey={0}) (cn=*{1}))",      // filter expression
		new Object[]{key, name},                // filter arguments
		null);					// default search controls

	    // Print the answer
	    printSearchEnumeration(answer);
	} catch (NamingException e) {
	    e.printStackTrace();
	}
    }

    public static void printSearchEnumeration(NamingEnumeration enum) {
	try {
	    while (enum.hasMore()) {
		SearchResult sr = (SearchResult)enum.next();
		System.out.println(">>>" + sr.getName());
		System.out.println(sr.getAttributes());
	    }
	} catch (NamingException e) {
	    e.printStackTrace();
	}
    }

}
